﻿/***********************************************************************
 合成フォントの名前を変更
    2024/04/02  version 0.24 Copyright 2024 CS5 Omachi
    使い方は→ https://cs5.xyz/script/renamecompositefont
 **********************************************************************/
//実行条件チェック
if (app.documents.length < 1) {
	alert("合成フォントを含むドキュメントを開いてください");
	exit();
}
var doc = app.documents[0];
if (doc.compositeFonts.length < 2) {
	alert("合成フォントがありません");
	exit();
}
//現在の合成フォント名を配列に入れる
var i, fno = [], fnn = [];
for (i = 1; i < doc.compositeFonts.length; i++) {
	fno.push(doc.compositeFonts[i].name);
}
fno.sort();

//ダイアログウィンドウの作成
var w = 520, h = 400, rh = 16;
var win = new Window("dialog", "ドキュメントの合成フォント名変更 Ver.0.24");
win.add("statictext", [0, 0, w, rh * 5], "新しい合成フォント名を入れてください。変更しない場合は空欄のままです。\
名前は28文字以内で、次の文字は使用できません。\" % & \( \) \* \. \/ : < > \? \[ \\ \] _ \{ \| \}\
新旧を含めて重複する名前は使用できません。\
詳しい使い方は→ https://cs5.xyz/script/renamecompositefont", {multiline:true});
var r = win.add("group");
r.alignment = "left";
r.add("statictext", [0, 0, w/2-10, rh], " ===== 現在の合成フォント名 ===== ");
r.add("statictext", [0, 0, w/2-10, rh], " ===== 新しい合成フォント名 ===== ");

var pnl = win.add("panel");
pnl.size = [w, h]; //別で指定しないとonShowでエラー
pnl.margins = 0;
var rs = pnl.add("group");
rs.orientation = "column";
rs.alignment = "left";
rs.maximumSize.height = fno.length * 100;

var i, st;
for (i = 0; i < fno.length; i++) {
	r = rs.add("group");
	r.orientation = "row";
	r.alignment = "left";
	r.margins = -4;
	st = r.add("statictext", [0, 0, w/2-20, rh*1.5], fno[i]);
	st.justify ="right";
	fnn[i] = r.add("edittext", [0, 0, w/2-20, rh*1.5]);
}

var bar = pnl.add("scrollbar");
bar.stepdelta = 10;
bar.maximumSize.height = pnl.maximumSize.height;
bar.onChanging = function () {
	rs.location.y = -1 * this.value;
}
win.onShow = function() {
	bar.size = [20, pnl.size.height];
	bar.location = [pnl.size.width - 20, 0];
	bar.maxvalue = rs.size.height - pnl.size.height + 20;
};

r = win.add("group");
var ybtn = r.add("button", undefined, "OK");
var cbtn = r.add("button", undefined, "キャンセル")
cbtn.onClick = function() {
	win.close(0);
}
ybtn.onClick = function() {
	var regchr = /[\"%&\(\)\*\.\/:<>\?\[\\\]_\{\|\}]/;
	var regstr = /(AUX|CON|NUL|PRN|COM[0-9]|LPT[0-9])/i;
	for (i = 0; i < fno.length; i++) {
		fnn[i].text = fnn[i].text.replace(/^\s+|\s+$/g, "");
		if (fnn[i].text == "") continue;
		if (regchr.test(fnn[i].text)) {
			alert("合成フォント名に使用できない文字が含まれています。");
			fnn[i].active = true;
			return;
		}
		if (regstr.test(fnn[i].text)) {
			alert("合成フォント名に使用できない名前があります。");
			fnn[i].active = true;
			return;
		}
		if (fnn[i].text.length > 28) {
			alert("合成フォント名は28文字以内にしてください。");
			fnn[i].active = true;
			return;
		}
	}
	//名前の重複チェック
	var chkObj = {};
	for (i = 0; i < fno.length; i++) {
		chkObj[fno[i]] = 1; //現在の合成フォント名をプロパティに設定
	}
	var incnt = fno.length;
	var rscnt;
	for (i = 0; i < fno.length; i++) {
		if (fnn[i].text != "") {
			chkObj[fnn[i].text] = 1; //新しい合成フォント名をプロパティに設定
			incnt++; //設定回数
		}
		//プロパティの数を数える
		rscnt = 0;
		for (var p in chkObj) rscnt++;
		//追加した回数とプロパティ数が異なれば重複している
		if (incnt != rscnt) {
			alert("合成フォント名が重複しています。");
			fnn[i].active = true;
			return;
		}
	}
	win.close(1);
}
if (win.show() !== 1) exit();

//名前変更
var j = 0;
for (i = 0; i < fno.length; i++) {
	if (fnn[i].text != "") {
		doc.compositeFonts.itemByName(fno[i]).name = fnn[i].text;
		j++;
	}
}
if (j > 0) {
	alert(j + "個の合成フォント名を変更しました。\
念のためInDesignの再起動をお勧めします。");
} else {
	alert("合成フォントの名前変更はありませんでした。");
}
